/*------------------------------------------------------------------------
    File        : PubSub/TTPubSubReceive.p
    Purpose     : Recieve a temp-table message to a Sonic Topic 
  ----------------------------------------------------------------------*/

{common/define-ttCust.i}

{common/connectToSonicBroker.i}

DEFINE VARIABLE hMsgConsumer AS HANDLE NO-UNDO.

/* Subscribe to SampleTopic1. Messages are handled by the
"messageHandler" internal procedure. */
RUN createMessageConsumer IN hSession (
  INPUT THIS-PROCEDURE,    /* this proc will handle the message  */
  INPUT "messageHandler",  /* name of internal procedure         */
  OUTPUT hMsgConsumer).    /* output the message consumer handle */
   
RUN subscribe IN hSession (
  INPUT "SampleTopic1", /* name of topic to subscribe to               */
  INPUT ?,              /* by default, the subscription is not durable */
  INPUT ?,              /* by defualt, no message selector             */
  INPUT FALSE,          /* receive copies of my own messages too       */
  INPUT hMsgConsumer).  /* handles the incoming messages               */

/* Start receiving messages */
RUN startReceiveMessages IN hSession.

DISPLAY "Waiting to receive Temp-Table message...".

/* Wait to receive the messages. */
WAIT-FOR u1 OF THIS-PROCEDURE.

/* Clean up - Delete the session */
RUN deleteSession IN hSession.


PROCEDURE messageHandler:
  /* will be invoked when a message is received */
  DEFINE INPUT  PARAMETER phMessage     AS HANDLE.
  DEFINE INPUT  PARAMETER phMsgConsumer AS HANDLE.
  DEFINE OUTPUT PARAMETER phReply       AS HANDLE.
  
  DEFINE VARIABLE hTempTable AS HANDLE NO-UNDO.
  
  /* if it's not a temp-table message, complain */
  IF DYNAMIC-FUNCTION("getMessageType" IN phMessage) <> "TemptableMessage" THEN
    MESSAGE "Received message is not a Temp--Table Message!"
      VIEW-AS ALERT-BOX.
  ELSE DO:
    /* extract the body of the temp-table message */   
    hTemptable = DYNAMIC-FUNCTION("getTempTable" IN phMessage,
      INPUT ?, INPUT ?, INPUT ?). /* just use all the defaults */

    /* process the message */  
    RUN displayTempTable (INPUT hTempTable).  
  END.

  /* clean up - delete the message */
  RUN deleteMessage IN phMessage.
  
  /* close this procedure */
  APPLY "U1" TO THIS-PROCEDURE.
END.


PROCEDURE displayTempTable:
  DEFINE INPUT PARAMETER httCust AS HANDLE .

  DEFINE VARIABLE hQuery AS HANDLE NO-UNDO.

  CREATE QUERY hQuery.
  hQuery:ADD-BUFFER(httCust:DEFAULT-BUFFER-HANDLE).
  hQuery:QUERY-PREPARE("FOR EACH ttCust").
  hQuery:QUERY-OPEN().
  
  repeat-displayRecords:
  REPEAT:
    hQuery:GET-NEXT().
    
    IF hQuery:QUERY-OFF-END THEN
      LEAVE repeat-displayRecords. 

    DISPLAY
      httCust::CustNum
      httCust::NAME    FORMAT "X(40)".
  END.
  
END PROCEDURE.